@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silclog.c,v 1.22 2002/03/28 16:09:22 priikone Exp $ */
+/* $Id: silclog.c,v 1.23 2002/03/28 17:12:43 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -158,7 +158,7 @@ static bool silc_log_reset(SilcLog log)
 SILC_TASK_CALLBACK(silc_log_fflush_callback)
 {
   unsigned int u;
-  if (silc_log_quick) {
+  if (!silc_log_quick) {
     silc_log_flush_all();
     SILC_FOREACH_LOG(u)
       silc_log_checksize(&silclogs[u]);
