@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcschedule.c,v 1.34 2002/03/20 10:30:06 priikone Exp $ */
+/* $Id: silcschedule.c,v 1.36 2002/04/11 19:42:03 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcschedule_i.h"
@@ -109,12 +109,12 @@ do {								\
 /* SILC Task object. Represents one task in the scheduler. */
 struct SilcTaskStruct {
   SilcUInt32 fd;
-  struct timeval timeout;
-  SilcTaskCallback callback;
-  void *context;
-  bool valid;
-  SilcTaskPriority priority;
-  SilcTaskType type;
+  SilcTaskCallback callback;	   /* Task callback */
+  void *context;		   /* Task callback context */
+  struct timeval timeout;	   /* Set for timeout tasks */
+  unsigned int valid : 1;	   /* Set when task is valid */
+  unsigned int priority : 2;	   /* Priority of the task */
+  unsigned int type : 5;           /* Type of the task */
 
   /* Pointers forming doubly linked circular list */
   struct SilcTaskStruct *next;
@@ -936,6 +936,7 @@ static void silc_task_queue_alloc(SilcTaskQueue *queue)
 static void silc_task_queue_free(SilcTaskQueue queue)
 {
   silc_mutex_free(queue->lock);
+  memset(queue, 'F', sizeof(*queue));
   silc_free(queue);
 }
 
@@ -1172,10 +1173,11 @@ static int silc_schedule_task_remove(SilcTaskQueue queue, SilcTask task)
     next = first;
 
     while(1) {
-      next = next->next;
-      silc_free(next->prev);
-      if (next == first)
+      old = next->next;
+      silc_free(next);
+      if (old == first)
 	break;
+      next = old;
     }
 
     queue->task = NULL;
