@@ -21,7 +21,7 @@
  *
  * DESCRIPTION
  *
- *    XXX
+ *    Utility functions.
  *
  ***/
 
@@ -76,58 +76,27 @@ char *silc_get_time();
  *
  * SYNOPSIS
  *
- *    char *silc_to_upper(char *string);
+ *    bool silc_to_upper(const char *string, char *dest, SilcUInt32 dest_size);
  *
  * DESCRIPTION
  *
  *    Converts string to capital characters.
  *
  ***/
-char *silc_to_upper(char *string);
+bool silc_to_upper(const char *string, char *dest, SilcUInt32 dest_size);
 
-/****f* silcutil/SilcUtilAPI/silc_encode_pem
+/****f* silcutil/SilcUtilAPI/silc_to_lower
  *
  * SYNOPSIS
  *
- *    char *silc_encode_pem(unsigned char *data, SilcUInt32 len);
+ *    bool silc_to_lower(const char *string, char *dest, SilcUInt32 dest_size);
  *
  * DESCRIPTION
  *
- *    Encodes data into PEM encoding. Returns NULL terminated PEM encoded
- *    data string. Note: This is originally public domain code and is
- *    still PD.
+ *    Converts string to capital characters.
  *
  ***/
-char *silc_encode_pem(unsigned char *data, SilcUInt32 len);
-
-/****f* silcutil/SilcUtilAPI/silc_encode_pem_file
- *
- * SYNOPSIS
- *
- *    char *silc_encode_pem_file(unsigned char *data, SilcUInt32 data_len);
- *
- * DESCRIPTION
- *
- *    Same as silc_encode_pem() but puts newline ('\n') every 72 characters.
- *
- ***/
-char *silc_encode_pem_file(unsigned char *data, SilcUInt32 data_len);
-
-/****f* silcutil/SilcUtilAPI/silc_decode_pem
- *
- * SYNOPSIS
- *
- *    unsigned char *silc_decode_pem(unsigned char *pem, SilcUInt32 pem_len,
- *                                   SilcUInt32 *ret_len);
- *
- * DESCRIPTION
- *
- *    Decodes PEM into data. Returns the decoded data. Note: This is
- *    originally public domain code and is still PD.
- *
- ***/
-unsigned char *silc_decode_pem(unsigned char *pem, SilcUInt32 pem_len,
-			       SilcUInt32 *ret_len);
+bool silc_to_lower(const char *string, char *dest, SilcUInt32 dest_size);
 
 /****f* silcutil/SilcUtilAPI/silc_parse_userfqdn
  *
@@ -264,6 +233,19 @@ SilcUInt32 silc_hash_ptr(void *key, void *user_context);
  ***/
 SilcUInt32 silc_hash_id(void *key, void *user_context);
 
+/****f* silcutil/SilcUtilAPI/silc_hash_client_id_hash
+ *
+ * SYNOPSIS
+ *
+ *    SilcUInt32 silc_hash_client_id_hash(void *key, void *user_context)
+ *
+ * DESCRIPTION
+ *
+ *    Hash Client ID's hash.
+ *
+ ***/
+SilcUInt32 silc_hash_client_id_hash(void *key, void *user_context);
+
 /****f* silcutil/SilcUtilAPI/silc_hash_data
  *
  * SYNOPSIS
@@ -573,4 +555,36 @@ char *silc_get_username();
  ***/
 char *silc_get_real_name();
 
+/****f* silcutil/SilcUtilAPI/silc_get_mode_list
+ *
+ * SYNOPSIS
+ *
+ *    bool silc_get_mode_list(SilcBuffer mode_list, SilcUInt32 mode_list_count,
+ *                            SilcUInt32 **list);
+ *
+ * DESCRIPTION
+ *
+ *    Returns modes from list of 32 bit MSB first order values that are
+ *    encoded one after the other in the `mode_list' into the `list'
+ *    array.  The caller must free the returned list.  Return FALSE if
+ *    there is error parsing the list.
+ *
+ ***/
+bool silc_get_mode_list(SilcBuffer mode_list, SilcUInt32 mode_list_count,
+			SilcUInt32 **list);
+
+
+/****f* silcutil/SilcUtilAPI/silc_get_status_message
+ *
+ * SYNOPSIS
+ *
+ *    char *silc_get_status_message(SilcStatus status)
+ *
+ * DESCRIPTION
+ *
+ *    Returns status message string
+ *
+ ***/
+char *silc_get_status_message(unsigned char status);
+
 #endif	/* !SILCUTIL_H */
