@@ -139,17 +139,17 @@ void silc_server_free_sock_user_data(SilcServer server,
 void silc_server_remove_from_channels(SilcServer server,
 				      SilcSocketConnection sock,
 				      SilcClientEntry client,
-				      int notify,
-				      char *signoff_message,
-				      int keygen);
-int silc_server_remove_from_one_channel(SilcServer server,
-					SilcSocketConnection sock,
-					SilcChannelEntry channel,
-					SilcClientEntry client,
-					int notify);
+				      bool notify,
+				      const char *signoff_message,
+				      bool keygen);
+bool silc_server_remove_from_one_channel(SilcServer server,
+					 SilcSocketConnection sock,
+					 SilcChannelEntry channel,
+					 SilcClientEntry client,
+					 bool notify);
 void silc_server_disconnect_remote(SilcServer server,
 				   SilcSocketConnection sock,
-				   const char *fmt, ...);
+				   SilcStatus status, ...);
 SilcChannelEntry silc_server_create_new_channel(SilcServer server,
 						SilcServerID *router_id,
 						char *cipher,
@@ -208,15 +208,26 @@ void silc_server_save_users_on_channel(SilcServer server,
 				       SilcBuffer user_list,
 				       SilcBuffer mode_list,
 				       SilcUInt32 user_count);
-SilcSocketConnection silc_server_get_client_route(SilcServer server,
-						  unsigned char *id_data,
-						  SilcUInt32 id_len,
-						  SilcClientID *client_id,
-						  SilcIDListData *idata);
+void silc_server_save_user_channels(SilcServer server,
+				    SilcSocketConnection sock,
+				    SilcClientEntry client,
+				    SilcBuffer channels,
+				    SilcBuffer channels_user_modes);
+SilcSocketConnection
+silc_server_get_client_route(SilcServer server,
+			     unsigned char *id_data,
+			     SilcUInt32 id_len,
+			     SilcClientID *client_id,
+			     SilcIDListData *idata,
+			     SilcClientEntry *client_entry);
 SilcBuffer silc_server_get_client_channel_list(SilcServer server,
-					       SilcClientEntry client);
+					       SilcClientEntry client,
+					       bool get_private,
+					       bool get_secret,
+					       SilcBuffer *user_mode_list);
 SilcClientEntry silc_server_get_client_resolve(SilcServer server,
 					       SilcClientID *client_id,
+					       bool always_resolve,
 					       bool *resolved);
 void silc_server_stderr(char *message);
 
